<?php

/*
U.S. Naval Observatory Earth Day/Night Viewer

Language: PHP v7.1.8
Author: Jay Tanner - 2018
www.NeoProgrammics.com

This program uses an on-line API (Application Programming Interface) provided
by USNO. The API provides the map based on the given date, time and time zone
entered in the interface.

The program simply displays a Day/Night image of the Earth  at any given moment
over the period from 1700 to 2100. The API may take several seconds to respond,
so be patient.   The map image comes from a U.S. Navy web server and is freely
made available to the public.

There are three Earth view modes:

Mode
 1    Rectangular Day/Night map
      The shadow movement cycles from right to left over 24 hours.

 2    Earth sphere viewed from sunrise side.
      At every point along the terminator shadow line, the sun is rising.

 3    Earth sphere viewed from sunset side.
      At every point along the terminator shadow line, the sun is setting.

The actual map size is 2048 x 1024 pixels for the rectangular map or 1024 x 1024
pixels for the spherical image. The program scales down the image for the browser,
but the image could possibly be viewed full-screen in a separate tab/window.

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.

*/

// -------------------------
// Prepare for cookie usage.
   ob_start();

// Define the program cookie name and set it to expire in 30 days.
   $CookieName = 'Earth_Day_Night_View_Settings';
   $SetToExpireIn30Days = time() + 30*86400;

// Set minimum and maximum year span to allow.   Any year outside this
// range will be considered an invalid year.   For the DE405 ephemeris,
// the valid year range is from 1600 to 2200 on the Gregorian calendar.
   $YMin = '1700';
   $YMax = '2100';

// Define TextArea text and background colors.
   $TxColor = 'black';
   $BgColor = 'LightYellow';

   $IMGMode = 1;
   $ViewText = '';

// Define script path and filename.
   $_SCRIPT_PATH_     = BaseName($_SERVER['SCRIPT_FILENAME']);
   $at = "&#97;&#116;"; $UTC = "&#85;&#84;&#67";
   $_RUN_THIS_SCRIPT_ = $_SERVER['SCRIPT_NAME'];

// Define internal document page title, HTML page heading text and revision date
   $_INTERNAL_TITLE_  = "U.S. Naval Observatory Day/Night Map";
   $_INTERFACE_TITLE_ = "<b style='font-size:16px;'>U.S. Naval Observatory Day/Night Earth View For Any Date and Time</b><br><span style='font-size:12px;'>Year Range &nbsp;=&nbsp; $YMin &nbsp;to&nbsp; $YMax &nbsp;=&nbsp; 500 years&nbsp; on the Gregorian Calendar</span>";
   $_REVISION_DATE_   = 'Revised: '. gmdate("l, F d, Y $at h:i A   $UTC", FileMTime($_SCRIPT_PATH_));


// Define status message to be displayed while working.
   $w = "            COMPUTING ... THIS COULD TAKE SEVERAL SECONDS.";
   $StatusText = "OutputText.innerHTML='$w';";

// Do this only if the [COMPUTE] button was NOT clicked.
   if (!isset($_POST['DayNightRectMap']) and !isset($_POST['SunriseSideViewButton']) and !isset($_POST['SunsetSideViewButton']))
      {
// IF the [COMPUTE] button was NOT clicked AND an active cookie is
// found, then read and restore the interface settings from it.
   if (isset($_COOKIE[$CookieName]))
  {
   $CookieDataString = $_COOKIE[$CookieName];
    list
   (
    $Y,
    $mn,
    $dn,
    $JG,
    $hhmmssLT,
    $hhmmTZ,
    $DTOnOff
   ) = preg_split("[\|]", $CookieDataString);

// Do some formatting
   $m = intval($mn);
   $d = intval($dn);
   $JG = (strtoupper(substr(trim($JG), 0,1)) == 'J')? 'J':'G';
   $DTOnOff = (substr(strtoupper(trim($DTOnOff)),0,2) == 'OF')? 'OFF':'ON';
   $dTBGColor = 'white';
   $DTOnOffBGColor = ($DTOnOff == 'OFF')? 'silver':'cyan';
   if ($DTOnOffBGColor == 'silver') {$dTBGColor = 'silver';}

  } // End of  if isset(_COOKIE[...])

   else

// SET THE DEFAULT INTERFACE STARTUP VALUES
// TO CURRENT SYSTEM DATE/TIME/ZONE.
 {
  $Y  = date('Y');
  $m  = date('n'); $mn  = sprintf("%02d", $m);
  $d  = date('j'); $dn  = sprintf("%02d", $d);
  $JG = 'G';

  $hhmmssLT = date('H:i:s');
  $hhmmTZ   = substr(Hours_to_HMS(date('Z')/3600, 0, '+', ':'),0,6);
  $DTOnOff  = 'ON';
  $DTOnOffBGColor = 'cyan';
  $dTBGColor = 'white';

  $hhmmssdT = '';

// Store interface settings in cookie.
   $CookieDataString = "$Y|$mn|$dn|$JG|$hhmmssLT|$hhmmTZ|$DTOnOff";
   setcookie ($CookieName, $CookieDataString, $SetToExpireIn30Days);
  }
      } // End of  if (!isset(_POST['ComputeButton']))


// Read values of all interface settings.
   if (isset($_POST['DayNightRectMap']) or isset($_POST['SunriseSideViewButton']) or isset($_POST['SunsetSideViewButton']))
{
   if (isset($_POST['DayNightRectMap']))      {$IMGMode = 1;    $ViewText = 'Earth - Day/Night Rectangular Map';}
   if (isset($_POST['SunriseSideViewButton'])){$IMGMode = 2;    $ViewText = 'Earth - Sunrise-Side View';}
   if (isset($_POST['SunsetSideViewButton'])) {$IMGMode = 3;    $ViewText = 'Earth - Sunset-Side View';}

   $Y  = @trim($_POST['Y']);  $Y = intval($Y);
   $mn = @trim($_POST['mn']); $m = intval($mn);
   $dn = @trim($_POST['dn']); $d = intval($dn);

   $JG = @trim($_POST['JG']);
   $JG = (strtoupper(substr(trim($JG), 0,1)) == 'J')? 'J':'G';

   $hhmmssLT = @trim($_POST['hhmmssLT']);
   $hhmmssLT = Num_to_HMS ($hhmmssLT);

   $hhmmTZ = @trim($_POST['hhmmTZ']); if ($hhmmTZ == '') {$hhmmTZ = '+00:00';}
   if (HMS_to_Hours($hhmmTZ) > 0 and (substr($hhmmTZ,0,1)) <> '+') {$hhmmTZ = "+$hhmmTZ";}
   $hhmmTZ = substr(Num_to_HMS ($hhmmTZ),0,6);
   $hhmmTZ = RTrim($hhmmTZ, ':');
   $hhmmTZ = (HMS_to_Hours($hhmmTZ) == 0)? '+00:00' : $hhmmTZ;

   $hhmmssdT = @trim($_POST['hhmmssdT']);

// Do some interface formatting.
   $mn = sprintf("%02d", $m);
   $dn = sprintf("%02d", $d);
   $hhmmssLT = Hours_to_HMS(HMS_to_Hours($hhmmssLT), 0, '', ':');

   $hhmmTZ = substr(Hours_to_HMS(HMS_to_Hours($hhmmTZ), 0, '+', ':'),0,6);

   $DTOnOff = @trim($_POST['DTOnOff']);
   $DTOnOff = (substr(strtoupper(trim($DTOnOff)),0,2) == 'OF')? 'OFF':'ON';

   $DTOnOffBGColor = ($DTOnOff == 'OFF')? 'silver':'cyan';
   $dTBGColor = ($DTOnOff == 'OFF')? 'silver':'white';

   if ($DTOnOffBGColor == 'silver') {$dTBGColor = 'silver';}

// Store interface settings in cookie.
   $CookieDataString = "$Y|$mn|$dn|$JG|$hhmmssLT|$hhmmTZ|$DTOnOff";
   setcookie ($CookieName, $CookieDataString, $SetToExpireIn30Days);
}

   $wYMin = ($JG == 'J')? -4713 : 1700;

   if ($JG == 'G')
  {
   $MinDateStr = Inv_JD_Num(JD_Num($wYMin, 11, 24, $JG), $JG, 3);
  }
 else
  {
   $MinDateStr = Inv_JD_Num(JD_Num($wYMin, 1, 1, $JG), $JG, 3);
  }

   $MaxDateStr = Inv_JD_Num(JD_Num($YMax, 12, 31, $JG), $JG, 3);

   if (intval($MinDateStr) > 1000) {$MinDateStr = str_replace('+', ' ', $MinDateStr);}





// Filter for date error. TRUE = Date is OK
// or FALSE = Bad date error.
   $DateFlag = Check_Date($Y, $m, $d, $JG);
   $YearFlag = ($Y < $YMin or $Y > $YMax)? FALSE:TRUE;

if ($DateFlag === FALSE or $YearFlag === FALSE)
{
   $TextArea2Text = $MmmStr  = $DoWStr = $URL = '';

   $DateTimeStringUT = $ViewText = '';

   $TxColor = 'white';   $BgColor = '#AA0000';

   $hhmmssDT = '-';  $hhmmssdT = '';
   $TScale = "<b style='color:red; font-size:20px;'>ERROR: INVALID DATE</b>";

   $dTBGColor = 'silver';

   if ($YearFlag === FALSE)
  {
   $JGStr  = ($JG == 'J')? 'Julian':'Gregorian';

   $TextArea1Text =
"DATE ERROR:

The given year ($Y) is not within the valid range from 1700 to 2100
on the $JGStr calendar.";
  }
   else
{
   $TextArea2Text = $ZeroNoteStr = '';
   $TScale = "<b style='color:red; font-size:20px;'>ERROR: INVALID DATE</b>";

   if ($YMin < 0) {$ZeroNoteStr = "For calendar years:  Neg|Pos = BC|AD\n\n";}

   $JGStr  = ($JG == 'J')? 'Julian':'Gregorian';

   $TextArea1Text =
"DATE ERROR:

The given date elements (Year Month Day) = ($Y $mn $dn) do not represent
any valid date within the allowed $JGStr calendar range.

----------------------
ALLOWED CALENDAR RANGE
$JGStr calendar   1700 Jan 01 Fri  to  2100 Dec 31 Fri

-------------------
VALID DATE ELEMENTS\n"
.$ZeroNoteStr.
"Year  $wYMin to $YMax on the $JGStr calendar.
Month    1 to 12 - For January to December.
Day      1 to 31 - Depending on the month and if or not it is a leap year.";
}

}

   else

// Do this if no date error detected.

{
// Compute JD Number for the given local date.
   $JDNumLT = JD_Num($Y, $m, $d, $JG);

// Compute local Julian Date for 00:00:00 of Date.
   $JD00LT = $JDNumLT - 0.5;

// Compute NASA Delta T estimate for local date
// if option is ON, then Delta T is applied.
// If option is OFF, then set Delta T to zero.
   if ($DTOnOff == 'ON')
      {
       if (@trim($hhmmssdT) == '')
          {$hhmmssDT = NASA_Delta_T_Poly($JD00LT, $JG, 3);}
       else
          {
           $hhmmssDT = Num_to_HMS($hhmmssdT);
           $hhmmssdT = Hours_to_HMS(HMS_to_Hours($hhmmssDT), 0, '+', ':');
          }
      }
   else
      {
       $hhmmssDT = '-';  $hhmmssdT = @$hhmmssdT;
      }



// Compute fractions of a day corresponding to LT, TZ and DT.
   $FracDayLT = HMS_to_Hours($hhmmssLT)/24;
   $FracDayTZ = HMS_to_Hours($hhmmTZ)/24;
   $FracDayDT = HMS_to_Hours($hhmmssDT)/24;

// Compute Julian Dates for LT, UT and TT scales.
   $JDLT = $JD00LT + $FracDayLT;
   $JDUT = $JDLT - $FracDayTZ;
   $JDTT = $JDUT + $FracDayDT;

// Do some formatting.
   $JDLT = sprintf("%7.8f", $JDLT);
   $JDUT = sprintf("%7.8f", $JDUT);
   $JDTT = sprintf("%7.8f", $JDTT);

// Compute equivalent times table for LT, UT and TT scales.
   $TimesTableText = Times_Table($Y, $m, $d, $hhmmssLT, $hhmmTZ, $JG);

// Get number of days in the given month.
   $mDays = Days_in_Month($Y, $m, $JG);

// Get 3-letter abbreviations for month and day of week names.
   $MmmStr = substr("JanFebMarAprMayJunJulAugSepOctNovDec", 3*($m-1), 3);
   $DoWStr = substr("SunMonTueWedThuFriSat", 3*(($JDNumLT+1) % 7), 3);

// Construct Date/Time string for UT.
   $DateTimeStringUT = str_replace('+', '', Inv_JD ($JDUT));



// #############################################################
// #############################################################
// #############################################################
// START OF MAIN COMPUTATIONS BLOCK USING JDXX AND OTHER VALUES.
// Y m mn d dn JDLT JDUT JDTT hhmmssLT hhmmTZ hhmmssDT ... etc.

// Construct times table corresponding to local time and zone.
   $TimesTableText = Times_Table($Y, $m, $d, $hhmmssLT, $hhmmTZ, 'G');
   $TimesTableText = str_replace('Year Convention: Neg|Pos = BC|AD', '', $TimesTableText);


// Get URL of NASA Day/Night map image.
   $URL = NASA_Day_Night_View ($Y, $m, $d, $hhmmssLT, $hhmmTZ, $IMGMode);

// Determine time scale symbol to use.
// Default TScale = 'UT'
   $TScale = 'UT';
   if ($hhmmTZ == '+00:00' and $DTOnOff == 'OFF') {$TScale = 'TT';}


// END OF MAIN COMPUTATIONS BLOCK
// #############################################################
// #############################################################
// #############################################################
















// Construct output text block 1 (upper).
// This shows the interface settings values.

   $TextArea1Text =
"$TimesTableText";


// Construct output text block 2.
   $TextArea2Text = '';

}


// Determine the number of text rows to use in the output TextAreas.
   $TextArea1Rows = 2 + substr_count($TextArea1Text, "\n");
   $TextArea2Rows = 2 + substr_count($TextArea2Text, "\n");

   if ($TextArea1Rows == 2) {$TextArea1Rows = 0;}





   function Strip_WSpace ($TextString)
{
   return preg_replace("/\s+/", " ", trim($TextString));
}


/*
   This function returns a link to the NASA API to display the
   Day/Night Earth map for the given date/time mode.
*/

   function NASA_Day_Night_View ($Yn, $mn, $dn, $LThhmm, $TZhhmm, $MapMode123=1)
{
   $Y = intval(trim($Yn));
   $m = intval(trim($mn));
   $d = intval(trim($dn));
   $hhmmLT = trim($LThhmm);
   $hhmmTZ = trim(trim($TZhhmm));

   $mode = intval(trim($MapMode123));

   $FracDayLT = HMS_to_Hours($hhmmLT) / 24;
   $FracDayTZ = HMS_to_Hours($hhmmTZ) / 24;
   $JDNumLT   = JD_Num($Y, $m, $d);
   $JD00LT    = $JDNumLT - 0.5;
   $JDLT      = $JD00LT + $FracDayLT;

   $JDUT = $JDLT - $FracDayTZ;
   $JDNumUT = floor($JDUT + 0.5);
   list($Y, $m, $d) = preg_split('[ ]', Inv_JD_Num ($JDNumUT));
   $hours = 24 * (($JDUT - 0.5) - floor($JDUT - 0.5));
   $hhmmssUT = Hours_to_HMS($hours, 0, '', ':');
   $hh_mm_ss = HMS_to_hh_mm_ss($hhmmssUT);
   list($hh, $mm, $ss) = preg_split('[ ]', $hh_mm_ss);
   $hh = intval($hh);
   $mm = sprintf("%02d", intval($mm));
   $ss = sprintf("%02d", intval($ss));

// Request sunset-side view.
   if ($mode == 3)
      {return "http://api.usno.navy.mil/imagery/earth.png?view=set&date=$m/$d/$Y&time=$hh:$mm:$ss";}

// Request sunrise-side view.
   if ($mode == 2)
      {return "http://api.usno.navy.mil/imagery/earth.png?view=rise&date=$m/$d/$Y&time=$hh:$mm:$ss";}

// Request rectangular Day/Night Map - Default mode
   return "http://api.usno.navy.mil/imagery/earth.png?date=$m/$d/$Y&time=$hh:$mm:$ss";

}






/*
   This function computes the time string (hh:mm) corresponding to
   the given fraction of a day argument.
*/

   function Frac_Day_to_HMS ($FracOfDay)
{
   $w = floatval(trim($FracOfDay));

   $sign = ($w < 0)? '-' : '';

   $hhmmss = $sign.Hours_to_HMS (24 * abs($w), 0, '', ':');

   return $hhmmss;
}



   function HMS_to_hh_mm_ss ($hhmmss)
{
   $w = trim($hhmmss);
   $w = HMS_to_Hours($w);
   $w = Hours_to_HMS($w, 0, '', ':');

   list($hh, $mm, $ss) = preg_split('[ ]', str_replace(':', ' ', $w));

   return ("$hh $mm $ss");
}





   function JD_Num ($Year, $Month, $Day, $JGMode='G')
{
   $Y = intval(trim($Year));
   $m = intval(trim($Month));
   $d = intval(trim($Day));
   $M = (strtoupper(substr(trim($JGMode),0,1)) == 'J')? 0 : 1;
   $a = floor((14 - $m) / 12);
   $b = (($Y < 0)? $Y+1 : $Y) - $a;
   $c = floor($b/100);
   $J = floor(30.6001*(12*$a + $m + 1))
      + floor(365.25*($b + 4716)) + $d - 1524
      + $M*(floor($c/4) - $c + 2);
   return ("$Y $m $d" <> Inv_JD_Num($J, $JGMode))? FALSE : $J;
}



/*
   This function return the calendar date elements (Y m d) corre-
   sponding to the given calendar mode and Julian Day number, or
   JD number, not to be confused with the Julian Date (JD).

   The JD Number is the serial number of the date on the calendar
   and is always an integer value.

   Mathematically, the Julian Day number is numerically equivalent
   to the Julian Date value  at 12:00:00 noon on the corresponding
   calendar date.

   JDNum  = Julian Day Number
   JGMode = 'J' or 'G' = Julian or Gregorian calendar mode

   RetMode123 = Return mode: 1=Default, 2 or 3
                1 = '1949 5 20'
                2 = '+1949 05 20'
                3 = '+1949 May 20 Fri'
*/

   function Inv_JD_Num ($JDNum, $JGMode='G', $RetMode123=1)
{
   $w = intval(trim($RetMode123)); $RMode = ($w < 1 or $w > 3)? 1 : $w;
   $w = trim($JDNum); if(!is_numeric($w)) {return FALSE;}
   $J = floor($w + 0.5); if($J < 0) {return FALSE;}
   $M = (strtoupper(substr(trim($JGMode),0,1)) == 'J')? 0:1;
   $e = $M*floor(($J - 1867216.25)/36524.25);
   $f = $J + $M*($e - floor($e/4) + 1);
   $g = $f + 1524;
   $h = floor(($g - 122.1)/365.25);
   $i = floor(365.25 * $h);
   $j = floor(($g - $i)/30.6001);
   $d = $g - $i - floor(30.6001 * $j);
   $m = $j - 12*floor($j/14) - 1;
   $w = $h + floor((14-$m)/12) - 4716;
   $Y = ($w > 0)? $w : $w-1;

   $w   = '';
   $Mmm = substr('JanFebMarAprMayJunJulAugSepOctNovDec', 3*($m-1), 3);
   $DoW = substr('SunMonTueWedThuFriSat', 3*(($J+1) % 7), 3);
   $Y   = substr('     '.sprintf("%+05d", $Y), -5);

   switch ($RMode)
  {
   case  1: $w = intval(trim($Y))." $m $d"; break;
   case  2: $w = "$Y ".sprintf("%02d", $m).' '.sprintf("%02d", $d); break;
   case  3: $w = "$Y $Mmm ".sprintf("%02d", $d)." $DoW"; break;
   default: $w = ''; break;
  }
   return $w;
}



   function Num_to_HMS ($NumHMSVal)
{
   $w = trim($NumHMSVal);

   if ($w == '') {return '00:00:00';}

// Return input unchanged if non-numeric argument
// rather than return an error message.
   if (!is_numeric($w)) {return $NumHMSVal;}

   $FirstChar = substr($w,0,1);

   $NumSign = (intval($w) < 0)? '-':'+';

// Remove any numerical (±) sign
// and work with absolute value.
   $w = str_replace($NumSign, '', $w);

// Construct (w) into a 6-digit integer.
   if (strlen($w) == 1) {$w = "0$w";}
   if (strlen($w) == 2) {$w = $w."0000";}
   if (strlen($w) == 3) {$w = "0$w";}
   if (strlen($w) == 4) {$w = $w."00";}
   if (strlen($w) == 5) {$w = "0$w";}

// Only keep (+) sign if it was in the original argument.
// Otherwise, a positive value will NOT be returned with
// a (+) sign. This does not effect the sign of negative
// values.
   if ($FirstChar <> '+' and $NumSign == '+') {$NumSign = '';}

// Done.  Return time equivalent with colons.
   return $NumSign.substr($w,0,2).':'.substr($w,2,2).':'.substr($w,4,2);

} // END OF  Num_to_HMS (...)



   function HMS_to_Hours ($hhmmss)
{

// -----------------------------------------------------------
// Read hour angle string argument and force to lower case in
// the event any (h m s :) symbols are used.

   $hms = strtolower(trim($hhmmss));

// ------------------------------------------------------
// Convert any (h m s :) symbols into spaces.

   $hms = str_replace('h', ' ', $hms);
   $hms = str_replace('m', ' ', $hms);
   $hms = str_replace('s', ' ', $hms);
   $hms = str_replace(':', ' ', $hms);


// ----------------------------------------------------
// Normalize the spacing and then split and extract the
// individual time string elements (hh, mm,ss).

   $hms = preg_replace("/\s+/", " ", trim($hms));
   $whms = preg_split("[ ]", $hms);
   $whmsCount = count($whms);
   $hh = ($whmsCount >= 1)? bcadd($whms[0],"0", 20) : "0";
   $mm = ($whmsCount >= 2)? bcadd($whms[1],"0", 20) : "0";
   $ss = ($whmsCount >= 3)? bcadd($whms[2],"0", 20) : "0";

// ------------------------------------------------
// Remember and then remove any numerical (±) sign.

   $NumSign = (substr($hhmmss,0,1) == '-')? '-' : '';
   $hh = str_replace('-', '', $hh);
   $hh = str_replace('+', '', $hh);

// -------------------------------------------------------------
// If original angle argument began with a + sign, then preserve
// it so that all returned positive results will have a + sign.
// Otherwise, positive results will NOT have a + sign.

   if (substr($hhmmss,0,1) == '+') {$NumSign = '+';}

// --------------------------------------
// Compute decimal hours value equivalent
// to the combined hh,mm,ss values.

   $w2 = bcadd(bcadd(bcmul($hh,'3600',20),bcmul($mm,'60',20),20),$ss,20);

// ------------------------------------------------------------
// If result equates to zero, then suppress any numerical sign.

   if (bccomp($w2, '0', 20) == 0) {$NumSign = '';}

// ----------------------------------------------------------
// Round off result to 16 decimals, recalling numerical sign.

   $w =  bcadd(bcdiv($w2,3600, 20), '0.00000000000000005', 16);

// Trim off any redundant zeros/decimal point.
   return $NumSign . rtrim(rtrim($w, '0'), '.');

} // End of  HMS_to_Hours(...)



   function Hours_to_HMS ($hours, $ssDecimals=0, $PosSignSymbol='', $SymbolsMode='h')
{

// ---------------------------
// Initialize symbol carriers.
   $_h_ = $_m_ = $_s_ = '';

   if (trim($PosSignSymbol) == '')  {$PosSignSymbol = FALSE;}

// --------------------------------------------------------------
// Remember original numerical sign and work with absolute value.
   $sign = ($hours < 0)? '-' : '';    $hours = abs($hours);

// -------------------------------------------------------
// Remember numerical sign to be restored on exit, if any.
//   if (($PosSignFlag === TRUE or $PosSignFlag == '+') and $sign == '')
//      {$sign = '+';}

   if (($PosSignSymbol === TRUE or $PosSignSymbol == '+') and $sign == '') {$sign = '+';}

// ---------------------------------------------------
// Compute time elements from absolute hours argument.
   $hh = floor($hours);
   $minutes = 60*($hours - $hh);    $mm = floor($minutes);
   $seconds = 60*($minutes - $mm);  $ss = sprintf("%1.3f",  $seconds);

// -------------------------
// Format the time elements.
   $hh = sprintf("%02d",  $hh);
   $mm = sprintf("%02d",  $mm);
   $ss = sprintf("%1.$ssDecimals" . "f", $ss);

// ----------------------------------
// Patch for that blasted 60s glitch.
   if ($ss == 60) {$ss = 0; $mm++;}
   if ($mm == 60) {$mm = 0; $hh++;}

   $hh = sprintf("%02d",  $hh);
   $mm = sprintf("%02d",  $mm);

   $ss = sprintf("%1.$ssDecimals" . "f", $ss);
   if ($ss < 10) {$ss = "0$ss";}

// -----------------------------------------------
// Attach optional (h m s :) symbols as indicated.
// Default = 'h'
   if ($SymbolsMode == 0 or $SymbolsMode == '')
      {$_h_  =  $_m_  =  $_s_  = ' ';}

   if ($SymbolsMode == 1 or $SymbolsMode == ':')
      {$_h_  =  ':';  $_m_  =  ':';  $_s_  = '';}

   if ($SymbolsMode == 2 or strtolower($SymbolsMode) == 'h')
      {$_h_  =  'h ';  $_m_  =  'm ';  $_s_  =  's';}

// Done.
   return "$sign$hh$_h_$mm$_m_$ss$_s_";

} // End of  Hours_to_HMS(...)




   function Check_Date ($Year, $Month, $Day, $JGMode='G')
{
   $Y = intval(trim($Year));
   $m = intval(trim($Month));
   $d = intval(trim($Day));
   $w = JD_Num($Y, $m, $d, $JGMode);
   return ($w === FALSE)? $w : TRUE;
}



   function Times_Table ($Year, $Month, $Day, $HHmmssLT='0', $HHmmTZ='0', $JGMode='G')
{
// These are GLOBALs because they are interface settings.
   global $DTOnOff, $hhmmssdT, $hhmmssDT;

   $Y  = intval(trim($Year));   $Y  = sprintf("%d",   $Y);
   $m  = intval(trim($Month));  $mn = sprintf("%02d", $m);
   $d  = intval(trim($Day));    $dn = sprintf("%02d", $d);
   $JG = (strtoupper(substr(trim($JGMode),0,1)) == 'J')? 'J':'G';

   $JDNumLT = JD_Num($Y, $m, $d, $JG);
   $hhmmssLT = Hours_to_HMS(HMS_to_Hours($HHmmssLT), 0,  '', ':');
   $hhmmTZ   = Hours_to_HMS(HMS_to_Hours($HHmmTZ),   0, '+', ':');
   $hhmmTZ   = substr($hhmmTZ, 0,6);

// $dTLineStr = "\nNASA Delta T   [OFF]                                     ";
   $dTLineStr = "";


   if ($DTOnOff == 'ON')
      {
       if ($hhmmssdT == '')
          {$hhmmssDT = NASA_Delta_T_Poly($JDNumLT, $JG, 3);}
       else
          {
          $hhmmssDT = Num_to_HMS($hhmmssdT);
          $hhmmssdT = Hours_to_HMS(HMS_to_Hours($hhmmssDT), 0, '+', ':');
//          $hhmmssdT = substr($hhmmssdT, 0,6);
          }
      }
   else
      {
       $hhmmssDT = '-';
      }

   $FracDayLT = HMS_to_Hours($hhmmssLT) / 24;
   $FracDayTZ = HMS_to_Hours($hhmmTZ)  / 24;
   $FracDayDT = HMS_to_Hours($hhmmssDT) / 24;

   $JD00LT = $JDNumLT - 0.5;
   $JDLT   = $JD00LT  + $FracDayLT;
   $JDUT   = $JDLT    - $FracDayTZ;
   $JDTT   = $JDUT    + $FracDayDT;

   $DateTimeStrLT = Inv_JD ($JDLT, $JG);
   $DateTimeStrUT = Inv_JD ($JDUT, $JG);
   $DateTimeStrTT = Inv_JD ($JDTT, $JG);

   if ($Y > 1000)
  {
   $DateTimeStrLT = str_replace ('+', ' ', $DateTimeStrLT);
   $DateTimeStrUT = str_replace ('+', ' ', $DateTimeStrUT);
   $DateTimeStrTT = str_replace ('+', ' ', $DateTimeStrTT);
  }

// Do some formatting for neatness.
   $JDNumLT = substr(floor($JDLT + 0.5)."       ", 0,7);
   $JDNumUT = substr(floor($JDUT + 0.5)."       ", 0,7);
   $JDNumTT = substr(floor($JDTT + 0.5)."       ", 0,7);

   $JDLT = sprintf("%7.8f", $JDLT);
   $JDLT = substr("                $JDLT", -16);
   $JDUT = sprintf("%7.8f", $JDUT);
   $JDUT = substr("                $JDUT", -16);
   $JDTT = sprintf("%7.8f", $JDTT);
   $JDTT = substr("                $JDTT", -16);

   $FracDayLT = sprintf("%1.8f",  $FracDayLT);
   $FracDayTZ = sprintf("%+1.8f", $FracDayTZ);
   $FracDayDT = sprintf("%+1.8f", $FracDayDT);

   $JGStr  = ($JG == 'J')? 'Julian Calendar   ':'Gregorian';
   $hhmmTZ = ($hhmmTZ == '00:00')? '+'."$hmmmTZ" : $hhmmTZ;

   if ($DTOnOff == 'OFF') {$whmsDT    = '    -    ';  $DTOnOffBGColor = $dTBGColor = 'silver';}
   if ($FracDayDT == 0)   {$hmsDT     = '+00:00:00';}
   if ($DTOnOff == 'ON')  {$dTLineStr = "\nDelta T                         $hhmmssDT DT       $FracDayDT";  $DTOnOffBGColor='cyan';  $dTBGColor = 'white';}

return
"                    $JGStr
                  Calendar Date
                 Year Mmm dn DoW    Time        Julian Date     JD  Number
=============== ===== === == === ===========  ================  ==========
Local Date/Time $DateTimeStrLT LT  $JDLT  $JDNumLT LT
Local Time Zone                 $hhmmTZ:00 TZ       $FracDayTZ
Universal Time  $DateTimeStrUT UT  $JDUT  $JDNumUT UT"
. $dTLineStr . "\nDynamical Time  $DateTimeStrTT TT  $JDTT  $JDNumTT TT";
}



   function Inv_JD ($JD, $JGMode='G')
{
   $J  = floatval(trim($JD));  $JDNum = floor($J + 0.5);
   $JG = (strtoupper(substr(trim($JGMode),0,1)) == 'J')? 'J':'G';

   $hhmmssLT = Hours_to_HMS(24*(($J - 0.5) - floor($J - 0.5)), 0, '', ':');

   list($Y, $m, $d) = preg_split("[ ]", Inv_JD_Num($JDNum, $JG));

// Do some formatting.
   $Y   = sprintf("%+05d", $Y);  $m = intval($m);  $d = intval($d);
   $Mmm = substr("JanFebMarAprMayJunJulAugSepOctNovDec", 3*($m - 1), 3);
    $dn = sprintf("%02d", $d);
   $DoW = substr("SunMonTueWedThuFriSat", 3*(($JDNum + 1) % 7), 3);

   return "$Y $Mmm $dn $DoW $hhmmssLT";
}


   function Days_in_Month ($Year, $Month, $JGMode='G')
{
   $Y = intval(trim($Year));  if ($Y < -4714 or $Y > 9999) {return FALSE;}
   $m = intval(trim($Month)); if ($m < 1 or $m > 12) {return FALSE;}
   $JG = (strtoupper(substr(trim($JGMode), 0,1)) == 'J')? 'J':'G';
   $mDays = substr("312831303130313130313031", 2*($m-1), 2);
   if (Is_Leap_Year($Y, $JG) and $m == 2) {return $mDays + 1;}
   return $mDays;
}


    function Is_Leap_Year ($Year, $JGMode='G')
{
   $Y = trim($Year);   $Y += ($Y < 0)? 1:0;
   $JG = (strtoupper(substr(trim($JGMode), 0,1)) == 'J')? 'J':'G';
   if ($JG == 'J' and $Y % 4 == 0) {return TRUE;}
   return ($Y % 4 == 0 and !($Y % 100 == 0 and $Y % 400 <> 0));
}



/*
   NASA Delta T Polynomial Expressions

   All integer-encoded partial dates MUST start with a (±) sign.
   Anything without a sign will be considered a Julian Date or a
   JD number.

   Return mode 1 (default) returns result resolved to nearest second (+444).
   Return mode 2 returns full polynomial result in seconds (444.413605555556).
   Return mode 3 returns ±HH:MM:SS string (+00:07:24).
   Return mode 4 returns ±h m s string (+00h 07m 24s).
   Return mode 5 returns Delta T as a fraction of a day.
*/

   function NASA_Delta_T_Poly ($JDTTArg, $JGMode='G', $RetMode12345=1)
{
// _________________________________________________
// Get date elements corresponding to JDTT argument.
// Only the (Y, m) values will be used.

   $JDTT    = trim($JDTTArg);
   $JDNumTT = floor($JDTT + 0.5);

   $JG = (strtoupper(substr(trim($JGMode),0,1)) == 'J')? 'J':'G';

   $w = trim($RetMode12345);
   if ($w == '' or intval($w) < 1) {$w = 1;}
   if (intval($w) > 5) {$w = 5;}
   $RetMode = $w;

// Determine (Y, m, d) values from JD Number and JG mode.
// Only the (Y, m) values will be used.
   list($Y, $m, $d) = preg_split("[ ]", Inv_JD_Num($JDNumTT, $JG));


// ___________________________________________
// ERROR if year value is invalid.  It must be
// in the range from -2000 to 3000.

   if ($Y < -2000 or $Y == 0 or $Y > 3000) {return FALSE;}

// _______________________________________________________
// ERROR if month value is outside the range from 1 to 12.

   if ($m < 1 or $m > 12) {return FALSE;}

/* ___________________________________________________________
   If a negative (BC) year is given , it is converted into the
   corresponding mathematical year by adding 1 to (Y).

   The year 100 BC (-100), becomes the mathematical year -99
   for proper use within internal computations.

   Compute working (y) value for mid-month (m).
*/
   $y = $Y + (($Y < 0)? 1:0) + ($m - 0.5)/12;

// _________________________________________________________________
// Perform NASA Delta T polynomial computations using (Y, y) values.

   $dTSec = FALSE;

   if ($Y >= -2000 and $Y < -500)
      {
       $u  = ($Y - 1820) / 100;

       $dTSec = 32*$u*$u - 20;
      }

   if ($Y >= -500 and $Y < 500)
      {
       $u  = $y/100;
       $u2 = $u * $u;
       $u3 = $u * $u2;
       $u4 = $u * $u3;
       $u5 = $u * $u4;
       $u6 = $u * $u5;

       $dTSec = 10583.6 - 1014.41*$u
                        + 33.78311*$u2
                        -  5.952053*$u3
                        -  0.1798452*$u4
                        +  0.022174192*$u5
                        +  0.0090316521*$u6;
      }


   if ($Y >= 500 and $Y < 1600)
      {
       $u  = ($y - 1000)/100;
       $u2 = $u * $u;
       $u3 = $u * $u2;
       $u4 = $u * $u3;
       $u5 = $u * $u4;
       $u6 = $u * $u5;

       $dTSec = 1574.2 - 556.01*$u
                       +  71.23472*$u2
                       +   0.319781*$u3
                       -   0.8503463*$u4
                       -   0.005050998*$u5
                       +   0.0083572073*$u6;
      }


   if ($Y >= 1600 and $Y < 1700)
      {
       $u  = $y - 1600;

      $dTSec = 120 - 0.9808*$u - 0.01532*$u*$u + $u*$u*$u/7129;
      }


   if ($Y >= 1700 and $Y < 1800)
      {
       $u  = $y - 1700;
       $u2 = $u * $u;
       $u3 = $u * $u2;
       $u4 = $u * $u3;

       $dTSec = 8.83 + 0.1603*$u
                     - 0.0059285*$u2
                     + 0.00013336*$u3
                     - $u4/1174000;
      }


   if ($Y >= 1800 and $Y < 1860)
      {
       $u  = $y - 1800;
       $u2 = $u * $u;
       $u3 = $u * $u2;
       $u4 = $u * $u3;
       $u5 = $u * $u4;
       $u6 = $u * $u5;
       $u7 = $u * $u6;

       $dTSec = 13.72 - 0.332447*$u
                      + 0.0068612*$u2
                      + 0.0041116*$u3
                      - 0.00037436*$u4
                      + 0.0000121272*$u5
                      - 0.0000001699*$u6
                      + 0.000000000875*$u7;
      }


   if ($Y >= 1860 and $Y < 1900)
      {
       $u  = $y - 1860;
       $u2 = $u * $u;
       $u3 = $u * $u2;
       $u4 = $u * $u3;
       $u5 = $u * $u4;

       $dTSec = 7.62 + 0.5737*$u
                     - 0.251754*$u2
                     + 0.01680668*$u3
                     - 0.0004473624*$u4
                     + $u5 / 233174;
      }


   if ($Y >= 1900 and $Y < 1920)
      {
       $u  = $y - 1900;
       $u2 = $u * $u;
       $u3 = $u * $u2;
       $u4 = $u * $u3;

       $dTSec = 1.494119*$u - 2.79
                            - 0.0598939*$u2
                            + 0.0061966*$u3
                            - 0.000197*$u4;
      }


   if ($Y >= 1920 and $Y < 1941)
      {
       $u  = $y - 1920;
       $u2 = $u * $u;
       $u3 = $u * $u2;

       $dTSec = 21.20 + 0.84493*$u - 0.076100*$u2 + 0.0020936*$u3;
      }


   if ($Y >= 1941 and $Y < 1961)
      {
       $u  = $y - 1950;
       $u2 = $u * $u;
       $u3 = $u * $u2;

       $dTSec = 29.07 + 0.407*$u - $u2/233 + $u3/2547;
      }


   if ($Y >= 1961 and $Y < 1986)
      {
       $u  = $y - 1975;
       $u2 = $u * $u;
       $u3 = $u * $u2;

       $dTSec = 45.45 + 1.067*$u - $u2/260 - $u3/718;
      }


   if ($Y >= 1986 and $Y < 2005)
      {
       $u  = $y - 2000;
       $u2 = $u * $u;
       $u3 = $u * $u2;
       $u4 = $u * $u3;
       $u5 = $u * $u4;

       $dTSec = 63.86 + 0.3345*$u
                      - 0.060374*$u2
                      + 0.0017275*$u3
                      + 0.000651814*$u4
                      + 0.00002373599*$u5;
      }


   if ($Y >= 2005 and $Y < 2050)
      {
       $u = $y - 2000;

       $dTSec = 62.92 + 0.32217*$u + 0.005589*$u*$u;
      }


   if ($Y >= 2050 and $Y <= 2150)
      {
       $u  = ($y - 1820) / 100;

       $dTSec = 32*$u*$u - 0.5628*(2150 - $y) - 20;
      }


   if ($Y > 2150 and $Y <= 3000)
      {
       $u  = ($y - 1820) / 100;

       $dTSec = 32*$u*$u - 20;
      }



// Format returned value according to return mode.
   if ($RetMode == 1) {return sprintf("%+d", round($dTSec, 0));}

   if ($RetMode == 2) {return $dTSec;} // Raw, No rounding

   if ($RetMode == 3 or $RetMode == 4)
      {
       $NumSign = ($dTSec < 0)? '-':'+';
       $hours   = abs($dTSec/3600);
       $hh      = floor($hours);
       $minutes = 60*($hours - $hh);
       $mm      = floor($minutes);
       $ss      = round(60*($minutes - $mm), 0);
       if ($ss == 60){$mm++; $ss=0;}
       if ($mm == 60){$hh++; $mm=0;}
       $hh      = sprintf("%02d", $hh);
       $mm      = sprintf("%02d", $mm);
       $ss      = sprintf("%02d", $ss);

       if ($RetMode == 3)
          {$hhmmssDT = "$NumSign$hh:$mm:$ss";}
       else
          {$hhmmssDT = "$NumSign$hh"."h $mm"."m $ss"."s";}

       return $hhmmssDT;
      }

   if ($RetMode == 5) {return sprintf("%+1.15f", round($dTSec, 0) / 86400);}

   return $dTSec; // = FALSE on error, if year is out of valid range.

} // END OF  NASA_Delta_T_Poly (...)



// Generate HTML web page to display computed results.
   print <<< _HTML

<!doctype html public "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">

<html>

<head>

<meta http-equiv="content-type" content="text/html; charset=UTF-8">

<meta name="robots"    content="noindex,nofollow">
<meta name="googlebot" content="noindex,nofollow">

<title>$_INTERNAL_TITLE_</title>

<script language="javascript">

function sleep_until (seconds) {
   var max_sec = new Date().getTime();
   while (new Date() < max_sec + seconds * 1000) {}
    return true;
}

</script>



<style type='text/css'>
BODY
{color:silver; background-color:black;  font-family:Verdana; font-size:14px}

TABLE
{font-size:12px; border: 1px solid black;}

TD
{color:black; background-color:white;  line-height:200%; font-size:12px; padding:6px; text-align:center;}

DIV
{color:black; background-color:white;  font-family:Verdana; text-align:justify; padding:4px; font-size:12px;}

PRE
{color:black; background-color:white;  font-family:monospace; font-size:14px; font-weight:bold; text-align:left; padding:4px;}

CODE
{font-family:monospace; font-size:15px; font-weight:bold;}

TEXTAREA
{color:black; background-color:white; font-family:monospace; font-size:15px; font-weight:bold; box-shadow:2px 3px 4px #888888; border-radius: 8px; border:1px solid black; padding:4px; white-space:pre;}

INPUT[type="text"]::-ms-clear {width:0; height:0;}

INPUT[type="text"]
{color:black; background-color:white; font-size:18px; font-family:monospace; font-weight:bold; text-align:center; box-shadow:2px 2px 3px #666666; border:2px solid black;}

INPUT[type="text"]:focus
{background-color:white; box-shadow:2px 2px 3px #666666; font-family:monospace; font-size:18px; border:2px solid red; text-align:center; font-weight:bold;}

INPUT[type="submit"]
{font-weight:bold;}

INPUT[type="submit"]:hover
{box-shadow:2px 2px 5px #555555; font-weight:bold;}

INPUT[type="checkbox"]:checked
{box-shadow:2px 2px 3px #555555;}

HR {background-color:black; height:2px; border:0px;}

A:link,  A:visited {background-color:transparent; color:blue; font-family:Verdana; font-size:12px; font-weight:bold; text-decoration:none; line-height:150%; padding:2px;}
A:hover, A:active  {background-color:transparent; color:red;  font-family:Verdana; font-size:12px; font-weight:bold; text-decoration:none; line-height:150%; padding:2px;}

::selection      {background-color:yellow; color:black;}  /* Only affects IE9+ and Safari     */
::-moz-selection {background-color:yellow; color:black;}  /* Only affects FireFox and Mozilla */
</style>


</head>

<body>

<form name="form1" method="post" action="">

<table width="800" border="0" cellspacing="1" cellpadding="3">

<tr><td colspan="9" style='color:white; background-color:#0000DD; border:2px solid white; border-radius:8px 8px 0px 0px'><b>$_INTERFACE_TITLE_</b></td></tr>


<td width='225'><b>Calendar Date</b><br>Year&nbsp;&nbsp;Month&nbsp;&nbsp;Day<br>
<input name='Y'  type='text' value="$Y"  size='6' maxlength='5'>
<input name='mn' type='text' value="$mn" size='3' maxlength='2'>
<input name='dn' type='text' value="$dn" size='3' maxlength='2'>
<input name='JG' type='hidden' value="$JG" size='2' maxlength='1' style='color:white; background-color:black;' readonly>
</td>


<td><b>Local&nbsp;Time</b><br>
&nbsp;&nbsp;&nbsp;00h &nbsp;to&nbsp; 24h<br>
<input name="hhmmssLT" type="text" value="$hhmmssLT" size="9" maxlength="8">
</td>

<td>
<b>Time&nbsp;Zone&nbsp;Offset</b><br>Neg&nbsp;|&nbsp;Pos&nbsp;&nbsp;=&nbsp;&nbsp;W&nbsp;|&nbsp;E<br>
<b style='font-size:15px;'>UT </b><input name="hhmmTZ" type="text" value="$hhmmTZ" size="7" maxlength="6">

<input name="hhmmssDT" style='color:white; background-color:black;' type="hidden" value="$hhmmssDT" size="10" maxlength="9" readonly>&nbsp;
<input name="hhmmssdT" type="hidden" value="$hhmmssdT" size="10" maxlength="9" style="color:black; background-color:$dTBGColor;">
<input name='DTOnOff' type='hidden' size='4' maxlength='3' value="$DTOnOff" style="color:black; background-color:$DTOnOffBGColor;">
</td>

</tr>

<tr><td colspan='9' style='color:white; background-color:black;' ><b style='color:white; background-color:black;'>&nbsp;NOTE:&nbsp;</b>&nbsp;Please be patient.&nbsp; This program may require several seconds to return the Earth image.
<br><br>
<input name="DayNightRectMap" type="submit" action="post" value=" Day/Night Rectangular Map">&nbsp;&nbsp;&nbsp;
<input name="SunriseSideViewButton" type="submit" action="post" value="Earth Sunrise Side View">&nbsp;&nbsp;
<input name="SunsetSideViewButton"  type="submit" action="post" value="Earth Sunset Side View">
</td></tr>

<tr><td colspan="7" ID="OutputText" style='color:white; background-color:black;'><b style='color:GreenYellow;'>U.S. Naval Observatory: &nbsp; $ViewText</b><br><b><code>$DateTimeStringUT $TScale</code></b></td></tr>

<tr><td colspan="7" style='border-radius:8px;'><img src="$URL" width='100%' style='border-radius:8px;'></td></tr>


<!--
<tr>
<td colspan="7" style='background-color:black;'>
<textarea name="OutputText1" style="font-size:18px; color:$TxColor; background-color:$BgColor; padding:6px; box-shadow:none; border:2px solid white;" cols="81" rows="$TextArea1Rows" OnDblClick="this.select();" OnMouseUp="return true;" readonly>
$TextArea1Text
</textarea>
</td>
</tr>
--->

<tr><td colspan="7" style='color:gray; background-color:black;'>Program by Jay Tanner - $_REVISION_DATE_ - Image Courtesy of USNO API</td></tr>

</table>
</form>
</body>

</html>

_HTML;






?>


